// The Virtual Light Company 1996
//
// From Chapter 20: Late Night VRML 2.0 and java
//
// This is the Filtering server starter class.

package filter;

import java.net.*;
import java.io.*;
import filter.*;

public class Filter extends Thread
{
	private static final int BUFFER_LENGTH = 1024;
	private int	_port;
	private boolean		server_available = false;
	private DatagramSocket	listen_socket = null;
	private FilterManager	manager = null;
	private UserServer	server;

	private ThreadGroup	thread_group;
	
	public Filter(int port, ThreadGroup tg, UserServer userver)
	{
		super(tg, "Filter Server Thread");

		thread_group = tg;

		_port = port;

		System.out.println("Starting Update Server on UDP port " + port);
		try
		{
			listen_socket = new DatagramSocket(port);
		}
		catch(IOException e)
		{
			System.err.println("Unable to open Filter socket " + e.getMessage());
			return;
		}

		manager = new FilterManager();

		server = userver;

		server_available = true;
	}

	public void run()
	{
		byte[]	buffer;
		DatagramPacket packet;
		FilterMessage a_filter;

		// if the server failed to start then exit and do nothing.
		if(!server_available)
			return;

		// tell the filter manager to start running
		manager.start();

		// now process the input.
		while(true)
		{
			buffer = new byte[BUFFER_LENGTH];
			packet = new DatagramPacket(buffer, BUFFER_LENGTH);

			try
			{
				listen_socket.receive(packet);
				Thread.yield();
			}
			catch(IOException e)
			{
				System.err.println("packet error");
			}

			a_filter = new FilterMessage(packet, thread_group, server);
			a_filter.run();
			manager.add(a_filter);
		}
	}
}
